#pragma once
#include <vulkan/vulkan.hpp>

class pipeline {
public:
    virtual ~pipeline() = default;

    virtual void add_binding(uint32_t binding, vk::DescriptorType descriptor_type, uint32_t descriptor_count, const vk::Sampler* immutable_samplers) = 0;

    void add_uniform_buffer(uint32_t binding, uint32_t descriptor_count = 1) {
        add_binding(binding, vk::DescriptorType::eUniformBuffer, descriptor_count, nullptr);
    }
    void add_storage_buffer(uint32_t binding, uint32_t descriptor_count = 1) {
        add_binding(binding, vk::DescriptorType::eStorageBuffer, descriptor_count, nullptr);
    }
    void add_sampled_image(uint32_t binding, uint32_t descriptor_count = 1, const vk::Sampler* immutable_samplers = nullptr) {
        add_binding(binding, vk::DescriptorType::eSampledImage, descriptor_count, immutable_samplers);
    }
    void add_storage_image(uint32_t binding, uint32_t descriptor_count = 1) {
        add_binding(binding, vk::DescriptorType::eStorageImage, descriptor_count, nullptr);
    }
    void add_input_attachment(uint32_t binding, uint32_t descriptor_count = 1) {
        add_binding(binding, vk::DescriptorType::eInputAttachment, descriptor_count, nullptr);
    }
    void add_sampler(uint32_t binding, uint32_t descriptor_count = 1, const vk::Sampler* immutable_samplers = nullptr) {
        add_binding(binding, vk::DescriptorType::eSampler, descriptor_count, immutable_samplers);
    }
    void add_combined_image_sampler(uint32_t binding, uint32_t descriptor_count = 1, const vk::Sampler* immutable_samplers = nullptr) {
        add_binding(binding, vk::DescriptorType::eCombinedImageSampler, descriptor_count, immutable_samplers);
    }

    void create_pipeline_layout();

    virtual void create() = 0;
    virtual void destroy() = 0;
    virtual void bind(vk::CommandBuffer command_buffer) const = 0;
    void create_pipeline_command() const;

    vk::Pipeline pipeline_;
    vk::PipelineCache pipeline_cache_;
    vk::PipelineLayout pipeline_layout_;
    std::vector<vk::DescriptorSetLayoutBinding> descriptor_set_layout_bindings_;
};
