//
// Created by 46944 on 2024/2/19.
//
#pragma once

#include <vulkan/vulkan.hpp>

#include "imgui_impl_vulkan.h"
#include "rhi/renderer.h"

static void check_vk_result(vk::Result err) {
    if (err == vk::Result::eSuccess)
        return;

    spdlog::error("[vulkan] Error: VkResult = {}", vk::to_string(err));
    abort();
}

static void check_vk_result(VkResult err) {
    if (err == VK_SUCCESS)
        return;
    if (err < 0) {
        spdlog::error("[vulkan] Error: VkResult = {}", err);
        abort();
    }
}

class renderer_vk : public renderer {
public:
    void pre_init() override;

    bool init(GLFWwindow* window_handle) override;

    void shutdown() override;

    std::shared_ptr<shader> load_shader(const std::string& entry_name) override;

    std::shared_ptr<pixel_shader_drawer> create_pixel_shader_drawer() override;

    void new_frame(GLFWwindow* window_handle) override;

    void end_frame(GLFWwindow* window_handle) override;

    void resize(int width, int height) override;

    std::shared_ptr<texture> create_texture(const unsigned char* data, int width, int height) override;

    std::shared_ptr<render_target> create_render_target(int width, int height, texture_format format) override;

    // Data
    vk::AllocationCallbacks* allocator = nullptr;
    vk::Instance instance = VK_NULL_HANDLE;
    vk::PhysicalDevice physical_device = VK_NULL_HANDLE;
    vk::Device device = VK_NULL_HANDLE;
    uint32_t queue_family = (uint32_t)-1;
    vk::Queue queue = VK_NULL_HANDLE;
    vk::DescriptorPool descriptor_pool = VK_NULL_HANDLE;
    vk::CommandPool command_pool = VK_NULL_HANDLE;

    ImGui_ImplVulkanH_Window main_window_data;
    int min_image_count = 2;
    bool swap_chain_rebuild = false;

protected:
    [[nodiscard]] vk::PhysicalDevice setup_vulkan_select_physical_device() const;

    void setup_vulkan(ImVector<const char*> instance_extensions);

    void setup_vulkan_window(VkSurfaceKHR surface, int width, int height);

    void cleanup_vulkan() const;

    void cleanup_vulkan_window();

    void frame_render(ImGui_ImplVulkanH_Window* wd, ImDrawData* draw_data);

    void frame_present(ImGui_ImplVulkanH_Window* wd);

private:
    void init_vulkan(GLFWwindow* window_handle);

    bool has_initialized_ = false;
};
